import * as puppeteer from 'puppeteer-core';
import * as action from '../actions';
import { ClickOptions, DeviceName, HoverOptions, KeyboardKey, KeyboardPressOptions, LaunchOptions, MinViewPort, PasteOptions, SelectOptionInfo, SelectOptions, TypeTextOptions, WaitOptions, WindowState } from '../actions';
import { SelectorController } from '../selector';
export declare type Story = (pptc: PuppeteerController) => Promise<void>;
export declare type StoryWithProps<T> = (pptc: PuppeteerController, props: T) => Promise<void>;
interface ExpectAssertionForAttribute {
    withValue: (attributeValue: string, options?: Partial<AssertOptions>) => PuppeteerController;
}
export interface ExpectAssertion {
    hasAttribute: (attributeName: string) => ExpectAssertionForAttribute;
    hasFocus: (options?: Partial<AssertOptions>) => PuppeteerController;
    hasClass: (className: string, options?: Partial<AssertOptions>) => PuppeteerController;
    hasExactValue: (value: string, options?: Partial<AssertOptions>) => PuppeteerController;
    hasText: (text: string, options?: Partial<AssertOptions>) => PuppeteerController;
    isDisabled: (options?: Partial<AssertOptions>) => PuppeteerController;
    isEnabled: (options?: Partial<AssertOptions>) => PuppeteerController;
    isVisible: (options?: Partial<AssertOptions>) => PuppeteerController;
    isNotVisible: (options?: Partial<AssertOptions>) => PuppeteerController;
}
export interface AssertOptions {
    /**
     * Defaults to 30000 milliseconds.
     *
     * @type {number}
     * @memberof AssertOptions
     */
    timeoutInMilliseconds: number;
    /**
     * time during which the Assert must give back the same result.
     * Defaults to 300 milliseconds.
     * You must not setup a duration < 100 milliseconds.
     * @type {number}
     * @memberof AssertOptions
     */
    stabilityInMilliseconds: number;
}
export interface WaitUntilOptions {
    /**
     * Defaults to 30000 milliseconds.
     *
     * @type {number}
     * @memberof WaitUntilOptions
     */
    timeoutInMilliseconds: number;
    /**
     * Time during which the callback must always return true.
     * Defaults to 300 milliseconds.
     * You must not setup a duration < 100 milliseconds.
     * @type {number}
     * @memberof AssertOptions
     */
    stabilityInMilliseconds: number;
    /**
     * Throw a timeout exception when the callback still returns false.
     * Defaults to false.
     * @type {boolean}
     * @memberof WaitUntilOptions
     */
    throwOnTimeout: boolean;
    /**
     * Output to the console all steps of the waiting mechanism.
     * Defaults to false.
     * Use this option when the waitUntil() method does not wait as expected.
     *
     * @type {boolean}
     * @memberof WaitUntilOptions
     */
    verbose: boolean;
}
export declare const defaultWaitUntilOptions: WaitUntilOptions;
export declare const defaultAssertOptions: AssertOptions;
export { ClickOptions, Device, DeviceName, HoverOptions, KeyboardKey, KeyboardPressOptions, LaunchOptions, mandatoryFullPageScreenshotOptions, MinViewPort, PasteOptions, SelectOptionInfo, SelectOptions, TypeTextOptions, WaitOptions, WindowState, } from '../actions';
export { NavigationOptions } from 'puppeteer-core';
export { SelectorController } from '../selector';
export { RequestInfo, ResponseInfo } from '../utils';
export declare class PuppeteerController implements PromiseLike<void> {
    /**
     * Create a controller instance by using an existing browser and page instance
     */
    constructor(browser?: puppeteer.Browser, page?: puppeteer.Page);
    then<TResult1 = void, TResult2 = never>(onfulfilled?: ((value: void) => TResult1 | PromiseLike<TResult1>) | null | undefined, onrejected?: ((reason: unknown) => TResult2 | PromiseLike<TResult2>) | null | undefined): Promise<TResult1 | TResult2>;
    private browser;
    private page;
    get currentPage(): puppeteer.Page | undefined;
    private _lastError?;
    get lastError(): Error | undefined;
    private isExecutingActions;
    private actions;
    private launchOptions;
    private pageErrors;
    getPageErrors(): Error[];
    clearPageErrors(): void;
    private failedRequests;
    getFailedRequests(): puppeteer.Request[];
    clearFailedRequests(): void;
    private sentRequests;
    getRequestsTo(url: string): puppeteer.Request[];
    getLastRequestTo(url: string): puppeteer.Request | undefined;
    clearRequestsTo(url: string): void;
    private executeActions;
    private launchAction;
    private startNewPage;
    getInstances(): [puppeteer.Browser, puppeteer.Page] | [undefined, undefined];
    initWith(options: Partial<LaunchOptions>): PuppeteerController;
    recordPageErrors(): PuppeteerController;
    recordFailedRequests(...additionalFailedStatus: number[]): PuppeteerController;
    recordRequestsTo(partialUrl: string): PuppeteerController;
    navigateTo(url: string, options?: Partial<puppeteer.NavigationOptions>): PuppeteerController;
    wait(durationInMilliseconds: number): PuppeteerController;
    click(selector: string, options?: ClickOptions): PuppeteerController;
    clear(selector: string, options?: ClickOptions): PuppeteerController;
    find(selector: string, waitOptions?: WaitOptions): {
        withText: (text: string) => {
            click: (clickOptions?: action.ClickOptions) => PuppeteerController;
        };
        withExactText: (text: string) => {
            click: (clickOptions?: action.ClickOptions) => PuppeteerController;
        };
    };
    hover(selector: string, options?: Partial<HoverOptions>): PuppeteerController;
    typeText(text: string, options?: TypeTextOptions): PuppeteerController;
    /**
     * Paste text in the element that is currently active (that has the focus) on the page.
     * If the element does not handle the 'paste' event through an explicit event handler,
     * then you must set the option {handlePasteEvent: true}.
     * When you set this option to true, the controller will automatically attach a basic 'paste' event handler.
     * @param {string} text
     * @param {PasteOptions} [options=defaultPasteOptions]
     * @returns {PuppeteerController}
     * @memberof PuppeteerController
     */
    pasteText(text: string, options?: PasteOptions): PuppeteerController;
    pressKey(key: KeyboardKey, options?: KeyboardPressOptions): PuppeteerController;
    select(...values: string[]): {
        in: (selector: string, options?: SelectOptions) => PuppeteerController;
    };
    close(): PuppeteerController;
    runStory<T>(story: Story | StoryWithProps<T>, param?: T): PuppeteerController;
    /**
     * Try to maximize the browser window size.
     * This method might not work on CI environment.
     * In this case setup the minViewPort parameter to ensure the page has a minimum width and height.
     * @param {MinViewPort} [minViewPort]
     * @returns {PuppeteerController}
     * @memberof PuppeteerController
     */
    withMaxSizeWindow(minViewPort?: MinViewPort): PuppeteerController;
    withCursor(): PuppeteerController;
    emulateDevice(deviceName: DeviceName): PuppeteerController;
    getCurrentUrl(): Promise<string>;
    getCurrentBrowserWindowState(): Promise<WindowState>;
    getValueOf(selector: string): Promise<string | undefined | null>;
    getClassListOf(selector: string): Promise<string[]>;
    getInnerTextOf(selector: string): Promise<string>;
    isChecked(selector: string): Promise<boolean>;
    hasFocus(selector: string): Promise<boolean>;
    isDisabled(selector: string): Promise<boolean>;
    isVisible(selector: string): Promise<boolean>;
    isNotVisible(selector: string): Promise<boolean>;
    hasClass(selector: string, className: string): Promise<boolean>;
    hasAttributeWithValue(selector: string, attributeName: string, attributeValue: string): Promise<boolean>;
    hasExactValue(selector: string, value: string): Promise<boolean>;
    hasText(selector: string, text: string): Promise<boolean>;
    getSelectedOptionOf(selector: string): Promise<SelectOptionInfo | null>;
    getAllOptionsOf(selector: string): Promise<SelectOptionInfo[]>;
    getComputedStyleOf(selector: string): Promise<CSSStyleDeclaration>;
    getClientRectangleOf(selector: string): Promise<ClientRect>;
    stringifyRequest(request: puppeteer.Request): Promise<string>;
    private safeExecutePredicate;
    private assertFor;
    expectThat(selector: string): ExpectAssertion;
    private waitFor;
    waitUntil(predicate: () => Promise<boolean>, options?: Partial<WaitUntilOptions>, errorMessage?: string | (() => Promise<string>)): PuppeteerController;
    private waitForStability;
    /**
     * Waits until the function getValue() returns the same result during a specified period of time.
     *
     * @param {(() => Promise<string | number>)} getValue async function that should return either a string or or number.
     * @param {Partial<WaitUntilOptions>} [options=defaultWaitUntilOptions]
     * @param {(string | (() => Promise<string>))} [errorMessage] error message to be returned when the throw option is activated
     * @returns {PuppeteerController}
     * @memberof PuppeteerController
     */
    waitForStabilityOf(getValue: () => Promise<string | number>, options?: Partial<WaitUntilOptions>, errorMessage?: string | (() => Promise<string>)): PuppeteerController;
    takeFullPageScreenshotAsBase64(options?: puppeteer.ScreenshotOptions): Promise<string>;
    /**
     * Create a Selector object to be able to target a DOM element
     * that is embedded in a complex dom hierarchy or dom array
     *
     * @param {string} selector
     * @returns {SelectorController}
     * @memberof PuppeteerController
     */
    selector(selector: string): SelectorController;
}
/**
 * cast input object as a PuppeteerController instance
 * usefull when such instance is store in an untyped context
 * @param pptcInstance : an untyped PuppeteerController instance
 */
export declare const cast: (pptcInstance: unknown) => PuppeteerController;
