import * as puppeteer from 'puppeteer-core';
import { PuppeteerController, ClickOptions } from '../controller';
import { HoverOptions } from '../actions';
export declare type Action = (handles: puppeteer.ElementHandle<Element>[]) => Promise<puppeteer.ElementHandle<Element>[]>;
export interface ActionInfoWithoutParam {
    name: 'parent' | 'unknown';
}
export interface ActionInfoWithSelector {
    name: 'querySelectorAllInPage' | 'find';
    selector: string;
}
export interface ActionInfoWithText {
    name: 'withText' | 'withValue';
    text: string;
}
export interface ActionInfoWithIndex {
    name: 'nth';
    index: number;
}
export declare type ActionInfo = ActionInfoWithoutParam | ActionInfoWithSelector | ActionInfoWithText | ActionInfoWithIndex;
export interface SelectorState {
    actions: ActionInfo[];
    chainingHistory: string;
}
export declare class SelectorController {
    private chainingHistory;
    private pptc;
    private actionInfos;
    private getActionFrom;
    private executeActions;
    /**
     * Executes the search.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     *
     * @returns {Promise<puppeteer.ElementHandle<Element>[]>} will return an empty array if no elements are found, will return all found elements otherwise.
     * @memberof SelectorController
     */
    getHandles(): Promise<puppeteer.ElementHandle<Element>[]>;
    /**
     * Executes the search and returns the first found element.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     *
     * @returns {Promise<puppeteer.ElementHandle<Element> | null>} will return null if no elements are found, will return first found element otherwise.
     * @memberof SelectorController
     */
    getFirstHandleOrNull(): Promise<puppeteer.ElementHandle<Element> | null>;
    /**
     * Gets the number of found elements.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     *
     * @returns {Promise<number>} will return 0 if no elements are found.
     * @memberof SelectorController
     */
    count(): Promise<number>;
    /**
     * Checks if the selector is visible.
     * If the selector targets multiple DOM elements, this check is done only on the first one found.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * So the visibilty status is the one known when executing this method.
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    isVisible(): Promise<boolean>;
    /**
     * Checks if the selector is disabled.
     * If the selector targets multiple DOM elements, this check is done only on the first one found.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * So the disability status is the one known when executing this method.
     *
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    isDisabled(): Promise<boolean>;
    /**
     * Checks if the selector is enabled.
     * If the selector targets multiple DOM elements, this check is done only on the first one found.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * So the enability status is the one known when executing this method.
     *
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    isEnabled(): Promise<boolean>;
    /**
     * Checks if selector has specified class.
     * If the selector targets multiple DOM elements, this check is done only on the first one found.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * @param {string} className
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    hasClass(className: string): Promise<boolean>;
    /**
     * Checks if selector exists.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * So the disability status is the one known when executing this method.
     *
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    exists(): Promise<boolean>;
    /**
     * Checks if selector does not exist.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * So the disability status is the one known when executing this method.
     *
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    doesNotExist(): Promise<boolean>;
    /**
     *
     */
    constructor(selector: string, pptc: PuppeteerController, stringifiedState?: string);
    toString(): string;
    private createSelectorFrom;
    find(selector: string): SelectorController;
    /**
     * Finds, from previous search, all elements whose innerText contains the specified text
     *
     * @param {string} text
     * @returns {SelectorController}
     * @memberof SelectorController
     */
    withText(text: string): SelectorController;
    /**
     * Finds, from previous search, all elements whose value contains the specified text
     *
     * @param {string} text
     * @returns {SelectorController}
     * @memberof SelectorController
     */
    withValue(text: string): SelectorController;
    parent(): SelectorController;
    /**
     * Takes the nth element found at the previous step
     *
     * @param {number} index : 1-based index
     * @returns {SelectorController}
     * @memberof SelectorController
     * @example
     * nth(1): take the first element found at previous step.
     * nth(-1): take the last element found at previous step.
     */
    nth(index: number): SelectorController;
    /**
     * Click on the selector.
     * This method automatically waits for the selector to appear in the DOM;
     * Then hover over the selector;
     * Then clicks on the selector.
     * @param {ClickOptions} [options=defaultClickOptions]
     * @returns {Promise<void>}
     * @memberof SelectorController
     */
    click(options?: Partial<ClickOptions>): Promise<void>;
    /**
     * Double Click on the selector.
     * This method automatically waits for the selector to appear in the DOM;
     * Then hover over the selector;
     * Then double-clicks on the selector.
     *
     * @param {Partial<ClickOptions>} [options=defaultClickOptions]
     * @returns {Promise<void>}
     * @memberof SelectorController
     */
    doubleClick(options?: Partial<ClickOptions>): Promise<void>;
    hover(options?: Partial<HoverOptions>): Promise<void>;
}
