"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const action = __importStar(require("../actions"));
const actions_1 = require("../actions");
const utils_1 = require("../utils");
class SelectorController {
    /**
     *
     */
    constructor(selector, pptc, stringifiedState) {
        this.chainingHistory = '';
        this.actionInfos = [];
        this.pptc = pptc;
        if (stringifiedState) {
            const state = JSON.parse(stringifiedState);
            this.chainingHistory = state.chainingHistory;
            this.actionInfos = state.actions;
            return;
        }
        this.chainingHistory = `selector(${selector})`;
        this.actionInfos.push({ name: 'querySelectorAllInPage', selector });
    }
    getActionFrom(actionInfo) {
        switch (actionInfo.name) {
            case 'querySelectorAllInPage':
                // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
                return () => action.querySelectorAllInPage(actionInfo.selector, this.pptc.currentPage);
            case 'find':
                // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
                return (handles) => action.querySelectorAllFromElements(actionInfo.selector, [...handles]);
            case 'nth':
                // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
                return (handles) => action.getNthElement(actionInfo.index, [...handles]);
            case 'parent':
                // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
                return (handles) => action.getParentsOf([...handles]);
            case 'withText':
                // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
                return (handles) => action.getElementsWithText(actionInfo.text, [...handles]);
            case 'withValue':
                // eslint-disable-next-line @typescript-eslint/explicit-function-return-type
                return (handles) => action.getElementsWithValue(actionInfo.text, [...handles]);
            default:
                throw new Error(`Action '${actionInfo.name}' is not yet implemented`);
        }
    }
    executeActions() {
        return __awaiter(this, void 0, void 0, function* () {
            let handles = [];
            for (let index = 0; index < this.actionInfos.length; index++) {
                const action = this.getActionFrom(this.actionInfos[index]);
                handles = yield action([...handles]);
            }
            return handles;
        });
    }
    /**
     * Executes the search.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     *
     * @returns {Promise<puppeteer.ElementHandle<Element>[]>} will return an empty array if no elements are found, will return all found elements otherwise.
     * @memberof SelectorController
     */
    getHandles() {
        return __awaiter(this, void 0, void 0, function* () {
            const handles = yield this.executeActions();
            return handles;
        });
    }
    /**
     * Executes the search and returns the first found element.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     *
     * @returns {Promise<puppeteer.ElementHandle<Element> | null>} will return null if no elements are found, will return first found element otherwise.
     * @memberof SelectorController
     */
    getFirstHandleOrNull() {
        return __awaiter(this, void 0, void 0, function* () {
            const handles = yield this.executeActions();
            if (handles.length === 0) {
                return null;
            }
            return handles[0];
        });
    }
    /**
     * Gets the number of found elements.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     *
     * @returns {Promise<number>} will return 0 if no elements are found.
     * @memberof SelectorController
     */
    count() {
        return __awaiter(this, void 0, void 0, function* () {
            const handles = yield this.executeActions();
            return handles.length;
        });
    }
    /**
     * Checks if the selector is visible.
     * If the selector targets multiple DOM elements, this check is done only on the first one found.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * So the visibilty status is the one known when executing this method.
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    isVisible() {
        return __awaiter(this, void 0, void 0, function* () {
            const handle = yield this.getFirstHandleOrNull();
            if (handle === null) {
                return false;
            }
            const isElementVisible = yield action.isHandleVisible(handle);
            return isElementVisible;
        });
    }
    /**
     * Checks if the selector is disabled.
     * If the selector targets multiple DOM elements, this check is done only on the first one found.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * So the disability status is the one known when executing this method.
     *
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    isDisabled() {
        return __awaiter(this, void 0, void 0, function* () {
            const handle = yield this.getFirstHandleOrNull();
            const isElementDisabled = yield action.isHandleDisabled(handle);
            return isElementDisabled;
        });
    }
    /**
     * Checks if the selector is enabled.
     * If the selector targets multiple DOM elements, this check is done only on the first one found.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * So the enability status is the one known when executing this method.
     *
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    isEnabled() {
        return __awaiter(this, void 0, void 0, function* () {
            const handle = yield this.getFirstHandleOrNull();
            const isElementEnabled = yield action.isHandleEnabled(handle);
            return isElementEnabled;
        });
    }
    /**
     * Checks if selector has specified class.
     * If the selector targets multiple DOM elements, this check is done only on the first one found.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * @param {string} className
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    hasClass(className) {
        return __awaiter(this, void 0, void 0, function* () {
            const handle = yield this.getFirstHandleOrNull();
            const classList = yield action.getClassListOfHandle(handle);
            const hasClass = classList.filter((c) => c === className).length > 0;
            return hasClass;
        });
    }
    /**
     * Checks if selector exists.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * So the disability status is the one known when executing this method.
     *
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    exists() {
        return __awaiter(this, void 0, void 0, function* () {
            const handle = yield this.getFirstHandleOrNull();
            if (handle === null) {
                return false;
            }
            return true;
        });
    }
    /**
     * Checks if selector does not exist.
     * The result may differ from one execution to another
     * especially if targeted element is rendered lately because its data is based on some backend response.
     * So the disability status is the one known when executing this method.
     *
     * @returns {Promise<boolean>}
     * @memberof SelectorController
     */
    doesNotExist() {
        return __awaiter(this, void 0, void 0, function* () {
            const handle = yield this.getFirstHandleOrNull();
            if (handle === null) {
                return true;
            }
            return false;
        });
    }
    toString() {
        return this.chainingHistory;
    }
    createSelectorFrom(selector, actions, chainingHistory) {
        const state = {
            actions,
            chainingHistory,
        };
        return new SelectorController(selector, this.pptc, JSON.stringify(state));
    }
    find(selector) {
        const actions = [...this.actionInfos];
        actions.push({ name: 'find', selector });
        const chainingHistory = `${this.chainingHistory}
  .find(${selector})`;
        return this.createSelectorFrom(selector, actions, chainingHistory);
    }
    /**
     * Finds, from previous search, all elements whose innerText contains the specified text
     *
     * @param {string} text
     * @returns {SelectorController}
     * @memberof SelectorController
     */
    withText(text) {
        const actions = [...this.actionInfos];
        actions.push({ name: 'withText', text });
        const chainingHistory = `${this.chainingHistory}
  .withText(${text})`;
        return this.createSelectorFrom(text, actions, chainingHistory);
    }
    /**
     * Finds, from previous search, all elements whose value contains the specified text
     *
     * @param {string} text
     * @returns {SelectorController}
     * @memberof SelectorController
     */
    withValue(text) {
        const actions = [...this.actionInfos];
        actions.push({ name: 'withValue', text });
        const chainingHistory = `${this.chainingHistory}
  .withValue(${text})`;
        return this.createSelectorFrom(text, actions, chainingHistory);
    }
    parent() {
        const actions = [...this.actionInfos];
        actions.push({ name: 'parent' });
        const chainingHistory = `${this.chainingHistory}
  .parent()`;
        return this.createSelectorFrom('', actions, chainingHistory);
    }
    /**
     * Takes the nth element found at the previous step
     *
     * @param {number} index : 1-based index
     * @returns {SelectorController}
     * @memberof SelectorController
     * @example
     * nth(1): take the first element found at previous step.
     * nth(-1): take the last element found at previous step.
     */
    nth(index) {
        const actions = [...this.actionInfos];
        actions.push({ name: 'nth', index });
        const chainingHistory = `${this.chainingHistory}
  .nth(${index})`;
        return this.createSelectorFrom('', actions, chainingHistory);
    }
    /**
     * Click on the selector.
     * This method automatically waits for the selector to appear in the DOM;
     * Then hover over the selector;
     * Then clicks on the selector.
     * @param {ClickOptions} [options=defaultClickOptions]
     * @returns {Promise<void>}
     * @memberof SelectorController
     */
    click(options = actions_1.defaultClickOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const clickOptions = Object.assign(Object.assign({}, actions_1.defaultClickOptions), options);
            yield this.pptc.waitUntil(() => this.exists(), {
                timeoutInMilliseconds: clickOptions.timeoutInMilliseconds,
                throwOnTimeout: true,
            }, `Cannot click on ${this.toString()} because this selector was not found in DOM`);
            const handle = yield this.getFirstHandleOrNull();
            yield action.scrollToHandle(handle);
            yield this.pptc.waitUntil(() => this.isVisible(), {
                timeoutInMilliseconds: clickOptions.timeoutInMilliseconds,
                throwOnTimeout: true,
            }, `Cannot click on ${this.toString()} because this selector is not visible`);
            yield action.waitUntilHandleDoesNotMove(handle, this.toString(), {
                timeoutInMilliseconds: clickOptions.timeoutInMilliseconds,
            });
            for (let index = 0; index < 3; index++) {
                yield utils_1.sleep(50);
                const clientRectangle = yield action.getClientRectangleOfHandle(handle);
                if (clientRectangle === null) {
                    continue;
                }
                const x = clientRectangle.left + clientRectangle.width / 2;
                const y = clientRectangle.top + clientRectangle.height / 2;
                this.pptc &&
                    this.pptc.currentPage &&
                    (yield this.pptc.currentPage.mouse.move(x, y, { steps: actions_1.defaultHoverOptions.steps }));
            }
            yield this.pptc.waitUntil(() => this.isEnabled(), {
                timeoutInMilliseconds: clickOptions.timeoutInMilliseconds,
                throwOnTimeout: true,
            }, `Cannot click on ${this.toString()} because this selector is disabled`);
            yield (handle === null || handle === void 0 ? void 0 : handle.click(clickOptions));
        });
    }
    /**
     * Double Click on the selector.
     * This method automatically waits for the selector to appear in the DOM;
     * Then hover over the selector;
     * Then double-clicks on the selector.
     *
     * @param {Partial<ClickOptions>} [options=defaultClickOptions]
     * @returns {Promise<void>}
     * @memberof SelectorController
     */
    doubleClick(options = actions_1.defaultClickOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const clickOptions = Object.assign(Object.assign({}, actions_1.defaultClickOptions), options);
            yield this.pptc.waitUntil(() => this.exists(), {
                timeoutInMilliseconds: clickOptions.timeoutInMilliseconds,
                throwOnTimeout: true,
            }, `Cannot double-click on ${this.toString()} because this selector was not found in DOM`);
            const handle = yield this.getFirstHandleOrNull();
            yield action.scrollToHandle(handle);
            yield this.pptc.waitUntil(() => this.isVisible(), {
                timeoutInMilliseconds: clickOptions.timeoutInMilliseconds,
                throwOnTimeout: true,
            }, `Cannot double-click on ${this.toString()} because this selector is not visible`);
            yield action.waitUntilHandleDoesNotMove(handle, this.toString(), {
                timeoutInMilliseconds: clickOptions.timeoutInMilliseconds,
            });
            for (let index = 0; index < 3; index++) {
                yield utils_1.sleep(50);
                const clientRectangle = yield action.getClientRectangleOfHandle(handle);
                if (clientRectangle === null) {
                    continue;
                }
                const x = clientRectangle.left + clientRectangle.width / 2;
                const y = clientRectangle.top + clientRectangle.height / 2;
                this.pptc &&
                    this.pptc.currentPage &&
                    (yield this.pptc.currentPage.mouse.move(x, y, { steps: actions_1.defaultHoverOptions.steps }));
            }
            yield this.pptc.waitUntil(() => this.isEnabled(), {
                timeoutInMilliseconds: clickOptions.timeoutInMilliseconds,
                throwOnTimeout: true,
            }, `Cannot double-click on ${this.toString()} because this selector is disabled`);
            const page = this.pptc.currentPage;
            if (!page) {
                throw new Error(`Cannot double-click on ${this.toString()} because no browser has been launched`);
            }
            if (page) {
                yield page.mouse.down({ clickCount: 1 });
                yield page.waitFor(clickOptions.delay || 0);
                yield page.mouse.down({ clickCount: 2 });
            }
        });
    }
    hover(options = actions_1.defaultHoverOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const hooverOptions = Object.assign(Object.assign({}, actions_1.defaultHoverOptions), options);
            yield this.pptc.waitUntil(() => this.exists(), {
                timeoutInMilliseconds: hooverOptions.timeoutInMilliseconds,
                throwOnTimeout: true,
            }, `Cannot hover on ${this.toString()} because this selector was not found in DOM`);
            const handle = yield this.getFirstHandleOrNull();
            yield action.scrollToHandle(handle);
            yield this.pptc.waitUntil(() => this.isVisible(), {
                timeoutInMilliseconds: hooverOptions.timeoutInMilliseconds,
                throwOnTimeout: true,
            }, `Cannot hover on ${this.toString()} because this selector is not visible`);
            yield action.waitUntilHandleDoesNotMove(handle, this.toString(), {
                timeoutInMilliseconds: hooverOptions.timeoutInMilliseconds,
            });
            for (let index = 0; index < 3; index++) {
                yield utils_1.sleep(50);
                const clientRectangle = yield action.getClientRectangleOfHandle(handle);
                if (clientRectangle === null) {
                    continue;
                }
                const x = clientRectangle.left + clientRectangle.width / 2;
                const y = clientRectangle.top + clientRectangle.height / 2;
                this.pptc &&
                    this.pptc.currentPage &&
                    (yield this.pptc.currentPage.mouse.move(x, y, { steps: hooverOptions.steps }));
            }
        });
    }
}
exports.SelectorController = SelectorController;
